// WARNING: This file is auto-generated and any changes to it will be overwritten
import lang.stride.*;
import java.util.ArrayList;

/**
 * Verwaltet einen ungerichteten, gewichteten Graphen mittels Adjazenzmatrix
 * @author Albert Wiedemann @version 1.0
 */
public class GraphMatrix
{
    /* Feld der Knoten des Graphen*/
    private ArrayList<Knoten> knoten;
    /* 2-dim Feld der Adjazenzmatrix*/
    private ArrayList<ArrayList<Integer>> matrix;
    /* Feld der Kantensymbole des Graphen*/
    private ArrayList<KantenSymbol> kanten;
    /* Die Warteliste (ToDo-Liste)*/
    private ArrayList<Integer> warteliste;
    /* Feld der fertigen Knoten*/
    private ArrayList<Integer> fertigeKnoten;

    /**
     * Baut die Datenstruktur auf
     */
    protected GraphMatrix()
    {
        knoten = new ArrayList < Knoten > ( );
        matrix = new ArrayList < ArrayList < Integer >> ( );
        kanten = new ArrayList < KantenSymbol > ( );
        warteliste = new ArrayList < Integer > ( );
        fertigeKnoten = new ArrayList < Integer > ( );
    }

    /**
     * Einfügen eines neuen Knoten in den Graphen @param bezeichner Bezeichner des neuen Knotens, der dem Graphen hinzugefügt wird. @param x x-Koordinate für die Anzeige des Knotens @param y y-Koordinate für die Anzeige des Knotens
     */
    protected void KnotenEinfügen(String bezeichner, int x, int y)
    {
        knoten . add ( new Knoten ( bezeichner , x , y ) );
        int index = 0;
        while (index < matrix . size ( )) {
            matrix . get ( index ) . add ( new Integer ( - 1 ) );
            index = index + 1;
        }
        matrix . add ( new ArrayList < Integer > ( ) );
        int index = 0;
        while (index < matrix . size ( )) {
            matrix . get ( matrix . size ( ) - 1 ) . add ( new Integer ( - 1 ) );
            index = index + 1;
        }
    }

    /**
     * Gibt den Knoten mit dem angegebenen Bezeichner zurück. Wenn ein Knoten mit diesem Bezeichner nicht bekannt ist wird null zurückgegeben @param bezeichner Bezeichner des Knoten der gesucht wird @return Referenz auf das Knotenobjekt oder null
     */
    private Knoten KnotenGeben(String bezeichner)
    {
        for (final Knoten k : knoten) {
            if (k . BezeichnerGeben ( ) . equals ( bezeichner )) {
                return k;
            }
        }
        return null;
    }

    /**
     * Gibt die interne Nummer des Knotens Wenn ein Knoten mit diesem Bezeichner nicht bekannt ist wird -1 zurückgegeben @param bezeichner Bezeichner des Knoten der gesucht wird @return Indexnummer des Knotens im Knotenfeld; 0 <= res <= knoten.size()-1 bzw. -1
     */
    protected int KnotenNummerGeben(String bezeichner)
    {
        int index = 0;
        while (index < knoten . size ( )) {
            if (knoten . get ( index ) . BezeichnerGeben ( ) . equals ( bezeichner )) {
                return index;
            }
            index = index + 1;
        }
        return - 1;
    }

    /**
     * Gibt die Bezeichnung eines Knotens mit der internen Knotennummer @param Indexnummer des Knotens im Knotenarray; 0<= x <= knoten.size() @return Bezeichner des Knotens
     */
    protected String KnotenBezeichnerGeben(int knotenNummer)
    {
        if (( knotenNummer < knoten . size ( ) ) && ( knotenNummer >= 0 )) {
            return knoten . get ( knotenNummer ) . BezeichnerGeben ( );
        }
        else {
            return "";
        }
    }

    /**
     * Einfügen einer Kante in den Graphen Eine Kante ist durch einen Anfangsknoten und einen Endknoten festgelegt und hat eine Gewichtung @param von Bezeichner des Anfangsknotens @param nach Bezeichner des Endknotens @param gewichtung Gewichtung der Kante als Ganzzahl @param gerichtet Gibt an, ob die Kante gerictet ist
     */
    protected void KanteEinfügen(String von, String nach, int gewichtung, boolean gerichtet)
    {
        int vonNummer;
        int nachNummer;
        vonNummer = KnotenNummerGeben ( von );
        nachNummer = KnotenNummerGeben ( nach );
        if (( vonNummer != - 1 ) && ( nachNummer != - 1 ) && ( vonNummer != nachNummer )) {
            matrix . get ( vonNummer ) . set ( nachNummer , gewichtung );
            if (! gerichtet) {
                matrix . get ( nachNummer ) . set ( vonNummer , gewichtung );
            }
            kanten . add ( new KantenSymbol ( knoten . get ( vonNummer ) . SymbolGeben ( ) , knoten . get ( nachNummer ) . SymbolGeben ( ) , gerichtet , "" + gewichtung , 3 , "blau" ) );
        }
    }

    /**
     * Gibt die Adjazenzmatrix des Graphen in der Konsole aus Nach Zeilen und Spalten formatiert Als Spaltenbreite wurde hier 4 Zeichen gewählt.
     */
    protected void Ausgeben()
    {
        /* Kopfzeile*/
        System . out . print ( " " );
        int i = 0;
        while (i < knoten . size ( )) {
            System . out . print ( ( knoten . get ( i ) . BezeichnerGeben ( ) + " " ) . substring ( 0 , 6 ) );
            i = i + 1;
        }
        System . out . println ( );
        int i = 0;
        while (i < knoten . size ( )) {
            System . out . print ( ( knoten . get ( i ) . BezeichnerGeben ( ) + " " ) . substring ( 0 , 4 ) );
            int j = 0;
            while (j < knoten . size ( )) {
                if (matrix . get ( i ) . get ( j ) != - 1) {
                    System . out . print ( ( matrix . get ( i ) . get ( j ) + " " ) . substring ( 0 , 4 ) );
                }
                else {
                    System . out . print ( " " );
                }
                j = j + 1;
            }
            System . out . println ( );
            i = i + 1;
        }
    }

    /**
     * Gibt die Anzahl der Knoten des Graphen zurück @return  Anzahl der Knoten
     */
    protected int KnotenAnzahlgeben()
    {
        return knoten . size ( );
    }

    /**
     * Gibt die Gewichtung einer Kante zurück Die Kante ist durch einen Anfangsknoten und einen Endknoten festgelegt @param von Bezeichner des Anfangsknotens @param nach Bezeichner des Endknotens @return Gewichtung der Kante
     */
    protected int KanteGewichtGeben(String von, String nach)
    {
        int vonNummer;
        int nachNummer;
        vonNummer = KnotenNummerGeben ( von );
        nachNummer = KnotenNummerGeben ( nach );
        if (( vonNummer != - 1 ) && ( nachNummer != - 1 )) {
            return matrix . get ( vonNummer ) . get ( nachNummer );
        }
        else {
            return - 1;
        }
    }

    /**
     * Löscht die Kanten und Knoten des Graphen Die Anzeige wird auch gelöscht
     */
    protected void ZurückSetzen()
    {
        for (final KantenSymbol k : kanten) {
            k . Entfernen ( );
        }
        for (final Knoten k : knoten) {
            k . SymbolGeben ( ) . Entfernen ( );
        }
        knoten . clear ( );
        kanten . clear ( );
        matrix . clear ( );
    }

    /**
     * Führt die Breitensuche vom Startknoten zum Zielknoten durch @param füllMenge die gewünschte Füllmenge @return wahr, wenn ein Knoten die gewünschte Füllmenge enthält
     */
    protected boolean BreitensucheAusführen(int füllMenge)
    {
        /* Entsprechend der Teilaufgabe b ergänzen*/
        return false;
    }
}
